@@ -257,6 +257,15 @@ int ldb_connect(struct ldb_context *ldb, const char *url,
 		return ret;
 	}
 
+	/*
+	 * Take a copy of the options.
+	 */
+	ldb->options = ldb_options_copy(ldb, options);
+	if (ldb->options == NULL && options != NULL) {
+		ldb_oom(ldb);
+		return LDB_ERR_OPERATIONS_ERROR;
+	}
+
 	ret = ldb_module_connect_backend(ldb, url, options, &ldb->modules);
 	if (ret != LDB_SUCCESS) {
 		return ret;
@@ -379,6 +388,7 @@ int ldb_transaction_start(struct ldb_context *ldb)
 				"ldb transaction start: %s (%d)",
 				ldb_strerror(status),
 				status);
+		ldb->transaction_active--;
 		}
 		if ((next_module && next_module->ldb->flags & LDB_FLG_ENABLE_TRACING)) {
 			ldb_debug(next_module->ldb, LDB_DEBUG_TRACE, "start ldb transaction error: %s",
@@ -424,6 +434,8 @@ int ldb_transaction_prepare_commit(struct ldb_context *ldb)
 		return LDB_SUCCESS;
 	}
 
+	ldb_reset_err_string(ldb);
+
 	status = next_module->ops->prepare_commit(next_module);
 	if (status != LDB_SUCCESS) {
 		ldb->transaction_active--;
@@ -495,9 +507,6 @@ int ldb_transaction_commit(struct ldb_context *ldb)
 			ldb_debug(next_module->ldb, LDB_DEBUG_TRACE, "commit ldb transaction error: %s",
 				  ldb_errstring(next_module->ldb));
 		}
-		/* cancel the transaction */
-		FIRST_OP(ldb, del_transaction);
-		next_module->ops->del_transaction(next_module);
 	}
 	return status;
 }
@@ -1088,7 +1097,6 @@ static int lock_search(struct ldb_module *lock_module, struct ldb_request *req)
 	if (ret == LDB_ERR_UNSUPPORTED_CRITICAL_EXTENSION) {
 		/* We might be talking LDAP */
 		ldb_reset_err_string(ldb);
-		ret = 0;
 		TALLOC_FREE(lock_context);
 
 		return ldb_next_request(lock_module, req);
