@@ -120,7 +120,7 @@ static LDAPMod **lldb_msg_to_mods(void *mem_ctx, const struct ldb_message *msg,
 			if (!mods[num_mods]->mod_vals.modv_bvals[j]) {
 				goto failed;
 			}
-			mods[num_mods]->mod_vals.modv_bvals[j]->bv_val = el->values[j].data;
+			mods[num_mods]->mod_vals.modv_bvals[j]->bv_val = (char *)el->values[j].data;
 			mods[num_mods]->mod_vals.modv_bvals[j]->bv_len = el->values[j].length;
 		}
 		mods[num_mods]->mod_vals.modv_bvals[j] = NULL;
@@ -141,7 +141,7 @@ static int lldb_add_msg_attr(struct ldb_context *ldb,
 			     struct ldb_message *msg, 
 			     const char *attr, struct berval **bval)
 {
-	int count, i;
+	int count, i, ret;
 	struct ldb_message_element *el;
 
 	count = ldap_count_values_len(bval);
@@ -150,25 +150,12 @@ static int lldb_add_msg_attr(struct ldb_context *ldb,
 		return -1;
 	}
 
-	el = talloc_realloc(msg, msg->elements, struct ldb_message_element, 
-			      msg->num_elements + 1);
-	if (!el) {
+	ret = ldb_msg_add_empty(msg, attr, 0, &el);
+	if (ret != LDB_SUCCESS) {
 		errno = ENOMEM;
 		return -1;
 	}
 
-	msg->elements = el;
-
-	el = &msg->elements[msg->num_elements];
-
-	el->name = talloc_strdup(msg->elements, attr);
-	if (!el->name) {
-		errno = ENOMEM;
-		return -1;
-	}
-	el->flags = 0;
-
-	el->num_values = 0;
 	el->values = talloc_array(msg->elements, struct ldb_val, count);
 	if (!el->values) {
 		errno = ENOMEM;
@@ -252,8 +239,11 @@ static int lldb_search(struct lldb_context *lldb_ac)
 		break;
 	}
 
-	tv.tv_sec = req->timeout;
+	tv.tv_sec = 0;
 	tv.tv_usec = 0;
+	if (req->timeout > 0) {
+		tv.tv_sec = req->timeout;
+	}
 
 	ret = ldap_search_ext(lldb->ldap, search_base, ldap_scope, 
 			      expression, 
@@ -455,7 +445,7 @@ static int lldb_del_trans(struct ldb_module *module)
 	return LDB_SUCCESS;
 }
 
-void lldb_request_done(struct lldb_context *ac,
+static void lldb_request_done(struct lldb_context *ac,
 			struct ldb_control **ctrls, int error)
 {
 	struct ldb_request *req;
@@ -489,7 +479,7 @@ static bool lldb_parse_result(struct lldb_context *ac, LDAPMessage *result)
 	char *errmsgp = NULL;
 	LDAPMessage *msg;
 	int type;
-	struct ldb_message *ldbmsg;
+	struct ldb_message *ldbmsg = NULL;
 	char *referral;
 	bool callback_failed;
 	bool request_done;
@@ -534,10 +524,6 @@ static bool lldb_parse_result(struct lldb_context *ac, LDAPMessage *result)
 				break;
 			}
 			ldap_memfree(dn);
-
-			ldbmsg->num_elements = 0;
-			ldbmsg->elements = NULL;
-
 			/* loop over all attributes */
 			for (attr=ldap_first_attribute(lldb->ldap, msg, &berptr);
 			     attr;
@@ -836,12 +822,13 @@ static int lldb_handle_request(struct ldb_module *module, struct ldb_request *re
 		return LDB_ERR_OPERATIONS_ERROR;
 	}
 
-
-	tv.tv_sec = req->starttime + req->timeout;
-	tv.tv_usec = 0;
-	te = tevent_add_timer(ev, ac, tv, lldb_timeout, ac);
-	if (NULL == te) {
-		return LDB_ERR_OPERATIONS_ERROR;
+	if (req->timeout > 0) {
+		tv.tv_sec = req->starttime + req->timeout;
+		tv.tv_usec = 0;
+		te = tevent_add_timer(ev, ac, tv, lldb_timeout, ac);
+		if (NULL == te) {
+			return LDB_ERR_OPERATIONS_ERROR;
+		}
 	}
 
 	return LDB_SUCCESS;
